/*
********************************************************************
**                                                                **
**	Tab Counter  -  Extension for Pale Moon                       **
**                                                                **
**	Last edit: January 18, 2020                                   **
**                                                                **
**	Release number: 3                                             **
**                                                                **
********************************************************************
*/
const {classes: Cc, interfaces: Ci, utils: Cu} = Components;
var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
var labelBackup = null;
var savedSTate = ""; // Currently applied settings.
var newState = ""; // New settings, applied/saved or not.
function savePrefs(saveNow) {
	if((newState == "") || (savedSTate == newState)) {
		return;
	};
	let formatStringPref = document.getElementById("formatStringPref");
	formatStringPref.valueFromPreferences = formatStringPref.value;
	let formatString = formatStringPref.valueFromPreferences;
	let sn = RegExp("%" + document.getElementById("bundle_tabcounterpm").getString("formatStringToolTip.number_s.key")).test(formatString);
	let vsn = RegExp("%" + document.getElementById("bundle_tabcounterpm").getString("formatStringToolTip.visible_number_s.key")).test(formatString);
	Cu.import("chrome://tabcounter-pm/content/tbcGlobal.jsm", savePrefs);
	if(vsn && !savePrefs.globalVal.getSetVal("tabGroupsExt", "getV")) { // get value.
		vsn = false;
	};
	if(document.getElementById("useFormatStringPref").value && (sn || vsn)) {
		document.getElementById("useGlobalCountsPref").valueFromPreferences = true;
		var totalCount_S = 0;
		var visCount_S = 0;
		let brWinEnum = wm.getEnumerator("navigator:browser");
		while(brWinEnum.hasMoreElements()) {
			let win = brWinEnum.getNext();
			if(win.gTabCounterPm) { // Collect locals, sum them...
				totalCount_S += win.gTabCounterPm.lastTotalCountCW;
				visCount_S += win.gTabCounterPm.lastVisCountCW;
			};
		};
		// ...and update globals with current values.
		savePrefs.globalVal.getSetVal("totalCount_S", "setV", totalCount_S); // set value.
		savePrefs.globalVal.getSetVal("visCount_S", "setV", visCount_S); // set value.
	} else {
		document.getElementById("useGlobalCountsPref").valueFromPreferences = false;
		savePrefs.globalVal.getSetVal("totalCount_S", "setV", 0); // set value.
		savePrefs.globalVal.getSetVal("visCount_S", "setV", 0); // set value.
	};
	let setMinWidthChkbx = document.getElementById("setMinWidth");
	if(setMinWidthChkbx.checked) {
		let minWidthPref = document.getElementById("minWidthPref");
		let tbcTlbrItem = wm.getMostRecentWindow("navigator:browser").document.getElementById("tabcounterToolbar_item");
		if(!minWidthPref.hasUserValue) {
			if(tbcTlbrItem) {
				let currentWidth = window.getComputedStyle(tbcTlbrItem).getPropertyValue("width");
				let currentWidthToInt = Math.round(parseFloat(currentWidth));
				// let currentWidthToInt = tbcTlbrItem.clientWidth; // Works the same.
				if(Number.isInteger(currentWidthToInt)) {
					minWidthPref.valueFromPreferences = currentWidthToInt;
				};
			};
			setMinWidthChkbx.label = setMinWidthChkbx.getAttribute("labelRem") + " ("+minWidthPref.valueFromPreferences+"px)";
			setMinWidthChkbx.parentNode.insertBefore(setMinWidthChkbx, setMinWidthChkbx.nextSibling);
		} else {
			minWidthPref.valueFromPreferences = minWidthPref.reset();
			setMinWidthChkbx.label = labelBackup;
			if(!tbcTlbrItem) { // Our toolbar item must be present on a toolbar to set again minimum width.
				setMinWidthChkbx.disabled = true;
			};
			setMinWidthChkbx.parentNode.insertBefore(setMinWidthChkbx, setMinWidthChkbx.nextSibling);
		};	// Required "fix", otherwise the accesskey is not shown after label toggling..
	};
	if(saveNow) {
		setMinWidthChkbx.checked = false;
		let prefs = document.getElementsByTagName("preference");
		for(let i=0; i<prefs.length; i++) {
			prefs[i].valueFromPreferences = prefs[i].value;
		};
		document.getElementById("tabCounterPm_prefs").getButton("extra1").disabled = true;
		savedSTate = newState;
	};
	var brWindowsEnum = wm.getEnumerator("navigator:browser");
	while(brWindowsEnum.hasMoreElements()) {
		let win = brWindowsEnum.getNext();
		if(win.gTabCounterPm) {
			win.gTabCounterPm.styleApplied = false;
			win.gTabCounterPm.updateToolbarItem("inProcess"); // Normally the argument is not needed here, added
			// just in case something goes wrong, to prevent multiple/infinite iterations in "updateToolbarItem".
		};
	};
};

function openLink(url) {
	window.close(); // Our options window is "modal", so it's pointless to stay open.
	let top = wm.getMostRecentWindow("navigator:browser");

	if(!top) {
		window.open(url, "", "");
    } else {
		let tBrowser = top.document.getElementById("content");

		// Is current tab blank. Different methods for determining depending on whether browser is
		// FF 1.5 or FF 1.0 based.
		let currBlank = (tBrowser &&
			(tBrowser.mCurrentTab.linkedBrowser && (tBrowser.mCurrentTab.linkedBrowser.contentDocument.location == "about:blank")) ||
			(!tBrowser.mCurrentTab.linkedBrowser && (tBrowser.mCurrentTab.label == "(Untitled)")));

		if(currBlank) {
			tBrowser.loadURI(url);
		} else {
			let tab = tBrowser.addTab(url);
			tBrowser.selectedTab = tab;
		};
	};
};

function toggleUseFormatString(useFormatString) {
	document.getElementById("optionTextGrid").hidden = useFormatString;
	document.getElementById("formatStringGrid").hidden = !useFormatString;
};

function useFormatStringCheckboxChanged(event) {
	toggleUseFormatString(event.target.checked);
};

/**
 * setMaxWindowSize
 *
 * Set the window size to the largest content that can be displayed by way of
 * the toggleUseFormatString() method.
 *
 * This is a pretty ugly hack, but it appears to be the easiest way to make
 * sure the window is big enough for whatever we will display.
 *
 * This method should only be called one time, from onLoad().
 * toggleUseFormatString() MUST be called after calling this method to set the
 * Options window to the correct state based on useFormatString.
 */
function setMaxWindowSize() {
	// Check the height when toggleUseFormatString is set to the true state:
	toggleUseFormatString(true);
	sizeToContent();
	var height_when_true = document.height;

	// Set toggleUseFormatString to the false state and resize to reflect that:
	toggleUseFormatString(false);
	sizeToContent();

	// If the window was bigger when toggleUseFormatString was in the true
	// state, resize the window to match the size in the that state.
	if(height_when_true > document.height) {
		toggleUseFormatString(true);
		sizeToContent();
	};
	
	// A bit of hack also here, to set the optimal width in our options window. The textboxes tend to get
	// an arbitrary width, even when empty, giving to our window a significant unused space at the side.
	// Don't like this..
	// Initially, we limit their width, just to be present for window's height calculation and after the final
	// action of "sizeToContent", where the size of the window has been finalized, we remove here the limitation,
	// letting textboxes to extend to the available space, without to affect the width of the window, which
	// is determined by the longest non-flexible element, like the length of labels in various locales.
	document.getElementById("col1").removeAttribute("maxwidth");
	document.getElementById("col2").removeAttribute("maxwidth");
	document.getElementById("col3").removeAttribute("maxwidth");
	
	// Finally, if it's needed we set the proper label for the "setMinWidth" checkbox according to "minWidthPref" preference.
	let minWidthPref = document.getElementById("minWidthPref");
	if(minWidthPref.hasUserValue) {
		let setMinWidthChkbx = document.getElementById("setMinWidth");
		setMinWidthChkbx.label = setMinWidthChkbx.getAttribute("labelRem") + " ("+minWidthPref.valueFromPreferences+"px)";
		setMinWidthChkbx.parentNode.insertBefore(setMinWidthChkbx, setMinWidthChkbx.nextSibling);
	};	// Required "fix", otherwise the accesskey is not shown after label toggling..
};

function getCurrentState(init) { // The produced string is used as a pseudo-hash of settings state.
	let str = "";
	let prefs = document.getElementsByTagName("preference");
	let checkboxState = document.getElementById("setMinWidth").hasAttribute("checked").toString();
	for(let pref of prefs) {
		if(pref.value !== undefined) {
			str += pref.value.toString();
		};
	};
	str += checkboxState;
	if(init) {
		savedSTate = str;
		return;
	};
	newState = str;
	return str;
};

function onLoad() {
	Cu.import("resource://gre/modules/AddonManager.jsm", onLoad);
	onLoad.AddonManager.getAddonByID("tabcounter-pm@chania.gr", function(tbc) {
		if(tbc.name) {
			document.title = tbc.name + " - " + document.title.replace(/\.\.\./g,"").replace(/…/g,"");
		}; // "Tab Counter - Preferences".
	});

	// Fix window size by giving explicit height to label.
	let elem = document.getElementById("cssInfo");
	elem.style.height = document.defaultView.getComputedStyle(elem, null).getPropertyValue("height");

	// Disable "Apply" button.
	document.getElementById("tabCounterPm_prefs").getButton("extra1").disabled = true;

	// Disable "setMinWidth" checkbox if it's in "set" state and our toolbar item is not present on a toolbar.
	if(!wm.getMostRecentWindow("navigator:browser").document.getElementById("tabcounterToolbar_item") &&
		!document.getElementById("minWidthPref").hasUserValue) { // State of checkbox = "set".
		document.getElementById("setMinWidth").disabled = true;
	};

	// There are two labels for the "setMinWidth" checkbox, which takes the proper one according to
	// "minWidthPref" preference. Initially we leave the one which most likely is the lengthiest in
	// most or all locales, regardless of "minWidthPref" preference, until the size of the window has
	// been finalized.
	// Also, this checkbox doesn't change something (it's not associated with a preference), to trigger
	// the "onchange" handler, so we have to toggle the state of "Apply" button in some other way.
	labelBackup = document.getElementById("setMinWidth").getAttribute("label");
	document.getElementById("setMinWidth").addEventListener("CheckboxStateChange", toggleApplyBtn, false);
	
	// Reveal hidden tooltips if "Tab Groups" extension is installed and active.
	Cu.import("chrome://tabcounter-pm/content/tbcGlobal.jsm", onLoad);
	if(onLoad.globalVal.getSetVal("tabGroupsExt", "getV")) { // get value.
		document.getElementById("key_sep").removeAttribute("hidden");
		document.getElementById("visible_after").removeAttribute("hidden");
		document.getElementById("visible_before").removeAttribute("hidden");
		document.getElementById("visible_current").removeAttribute("hidden");
		document.getElementById("visible_number").removeAttribute("hidden");
		document.getElementById("visible_number_all").removeAttribute("hidden");
	};

	// Update tooltip localization.
	localizeTooltips();

	// Correctly enable/disable the labelling fields.
	let useFormatStringCheckbox = document.getElementById("useFormatStringCheckbox");
	useFormatStringCheckbox.addEventListener("CheckboxStateChange", useFormatStringCheckboxChanged, false);
	setMaxWindowSize();
	toggleUseFormatString(useFormatStringCheckbox.checked);

	let acceptBtn = document.documentElement.getButton("accept");
	// We need the "accept" button always, even in Linux/Mac, because otherwise, any pending changes are
	// applied on exit only if the window is closed with enter. The same applies also for our routine to
	// update our toolbar item.
	if(acceptBtn.disabled) { // Mac OS.
		acceptBtn.removeAttribute("disabled");
	};
	if(acceptBtn.hidden) { // Linux/Mac OS.
		acceptBtn.removeAttribute("hidden");
	};
	let cancelBtn = document.documentElement.getButton("cancel");
	if(cancelBtn.hidden) { // Mac OS, it might be useful for some.
		cancelBtn.removeAttribute("hidden");
	};

	// Focus on first text field.
	document.getElementById("fontColor").focus();

	// Get and store the current settings state.
	getCurrentState(true);
};

function toggleApplyBtn() {
	// Enables the "Apply" button when there are changes in settings.
	// Disables the button if the changes are reverted back to the currently saved state.
	let applyBtn = document.documentElement.getButton("extra1");
	getCurrentState() !== savedSTate ? applyBtn.disabled = false : applyBtn.disabled = true;
};

function localizeTooltips() {
	let mBundle = document.getElementById("bundle_tabcounterpm");
	document.getElementById("after").value = "%" + mBundle.getString("formatStringToolTip.after.key") + document.getElementById("after").value;
	document.getElementById("before").value = "%" + mBundle.getString("formatStringToolTip.before.key") + document.getElementById("before").value;
	document.getElementById("current").value = "%" + mBundle.getString("formatStringToolTip.current.key") + document.getElementById("current").value;
	document.getElementById("number").value = "%" + mBundle.getString("formatStringToolTip.number.key") + document.getElementById("number").value;
	document.getElementById("number_all").value = "%" + mBundle.getString("formatStringToolTip.number_s.key") + document.getElementById("number_all").value;
	document.getElementById("visible_after").value = "%" + mBundle.getString("formatStringToolTip.visible_after.key") + document.getElementById("visible_after").value;
	document.getElementById("visible_before").value = "%" + mBundle.getString("formatStringToolTip.visible_before.key") + document.getElementById("visible_before").value;
	document.getElementById("visible_current").value = "%" + mBundle.getString("formatStringToolTip.visible_current.key") + document.getElementById("visible_current").value;
	document.getElementById("visible_number").value = "%" + mBundle.getString("formatStringToolTip.visible_number.key") + document.getElementById("visible_number").value;
	document.getElementById("visible_number_all").value = "%" + mBundle.getString("formatStringToolTip.visible_number_s.key") + document.getElementById("visible_number_all").value;
};
