/*
********************************************************************
**                                                                **
**	Tab Counter  -  Extension for Pale Moon                       **
**                                                                **
**	Last edit: September 13, 2018                                 **
**                                                                **
**	Release number: 1                                             **
**                                                                **
********************************************************************
*/
const {classes: Cc, interfaces: Ci, utils: Cu} = Components;

var overlay_loaded = false;
var windows_already_loaded = [];
var overlay = {};

/**
 * Provesses overlay document data and initializes overlay property.
 */
function processOverlay(/**Element*/ root) {
	// Remove whitespace text nodes.
	let walker = root.ownerDocument.createTreeWalker(
		root, Ci.nsIDOMNodeFilter.SHOW_TEXT,
		function(node) !/\S/.test(node.nodeValue), false
	);
	let whitespaceNodes = [];
	while(walker.nextNode()) {
		whitespaceNodes.push(walker.currentNode);
	};
	for(let i = 0; i < whitespaceNodes.length; i++) {
		whitespaceNodes[i].parentNode.removeChild(whitespaceNodes[i]);
	};
	// Put overlay elements into appropriate fields.
	while(root.firstElementChild) {
		let child = root.firstElementChild;
		if(child.getAttribute("id")) {
			overlay[child.getAttribute("id")] = child;
		};
		root.removeChild(child);
	};

	overlay_loaded = true;

	// Apply overlay to windows that already loaded.
	while(windows_already_loaded.length) {
		exports.applyOverlay(windows_already_loaded.pop(), "onEnabling");
	};
};

exports.loadOverlay = function() {
	let request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
	request.mozBackgroundRequest = true;
	request.addEventListener("load", function onLoad(event) {
		request.removeEventListener("load", onLoad, false);
		if(disabled) {
			return;
		};
		processOverlay(request.responseXML.documentElement);
	}.bind(this), false);
	request.open("GET", "chrome://tabcounterpm/content/tabcounter.xul");
	request.send(null);
};

exports.applyOverlay = function(window, aParam) {
	let toolbarElem = ["toolbaritem"];
	if(overlay_loaded) {
		// Add non speical items to window.
		for(let id in overlay) {
			// Toolbar item needs to be added manually.
			if(id == "BrowserToolbarPalette") {
				for(let i=0; i < overlay[id].childNodes.length; i++) {
					if(toolbarElem.indexOf(overlay[id].childNodes[i].nodeName.toLowerCase()) != -1) {
						restorePosition(window.document, overlay[id].childNodes[i].cloneNode(true));
					};
				};
			} else { // If normal element, just add it's children to the element.
				let elem = window.document.getElementById(id);
				if(elem) {
					for(let i=0; i < overlay[id].childNodes.length; i++) {
						elem.appendChild(overlay[id].childNodes[i].cloneNode(true));
					};
				};
			};
		};
		if(aParam == "onEnabling") {
			if(windows_already_loaded.length == 0) { // Last overlay, enable ability to update other windows,
			// just before run the script.
			// On extension's install/enable, only the last window will run its iterator to update the global
			// counts in other already open windows.
				Cu.import("chrome://tabcounter-pm/content/tbcGlobal.jsm", this);
				this.globalVal.getSetVal("canUpdateOtherWin", "setT"); // set true.
				
				// "domReadyWin": info in "tbcGlobal.jsm".
				this.globalVal.getSetVal("domReadyWin", "setV", // set value.
					this.globalVal.getSetVal("openWindows", "getV")); // get value.
			};
		};

		// Run script.
		try{
			Services.scriptloader.loadSubScript("chrome://tabcounterpm/content/tabcounter.js", window);
		} catch(ex) {
			Cu.reportError("Tab Counter exception: " + ex);
		};
		if(aParam == "onEnabling") {
			if(windows_already_loaded.length == 0) { // Last overlay loaded, restore "tbcEnabled" to default.
				this.globalVal.getSetVal("tbcEnabled", "setF"); // set false.
			};
		};

		function removeOverlay(winClose) {
			window.gTabCounterPm.uninitialize(winClose);

			// Try to find and remove our toolbar item from the toolbar palette.
			let toolbox = window.document.getElementById("navigator-toolbox");
			for(let childElem = toolbox.palette.firstElementChild; childElem; childElem = childElem.nextElementSibling) {
				if(childElem.id == "tabcounterToolbar_item") {
					childElem.parentNode.removeChild(childElem);
				};
			};
			for(let id in overlay) {
				for(let i=0; i < overlay[id].childNodes.length; i++) {
					let childElem = window.document.getElementById(overlay[id].childNodes[i].getAttribute("id"));
					if(childElem) {
						childElem.parentNode.removeChild(childElem);
					};
				};
			};
			window.gTabCounterPm = null;
		};
		unload(removeOverlay, window);
	} else {
		// Window loaded before we were ready so process it when finished loading overlay.
		windows_already_loaded.push(window);
	};
};
